/* ColView.cpp
	Copyright (C) 1996-99, Seth Flaxman (seth@abisoft.com)
	Permission to use, copy, modify, and distribute this software for any
	purpose without fee is hereby granted, provided that this entire notice
	is included in all copies of any software which is or includes a copy
	or modification of this software and in all copies of the supporting
	documentation for such software.
	THIS SOFTWARE IS BEING PROVIDED "AS IS", WITHOUT ANY EXPRESS OR IMPLIED
	WARRANTY.  IN PARTICULAR, THE AUTHOR DOES NOT MAKE ANY
	REPRESENTATION OR WARRANTY OF ANY KIND CONCERNING THE MERCHANTABILITY
	OF THIS SOFTWARE OR ITS FITNESS FOR ANY PARTICULAR PURPOSE.
*/
#include "ColView.h"
#include <stdio.h>
#include <Screen.h>
#include <Roster.h>
#include <Path.h>
#include <FindDirectory.h>
#include <Directory.h>
#include <Application.h>
#include <string.h>

const uint8 COLOR_BROADCAST = true;

ColView::ColView(BRect rect, char *name)
		: BView(rect, name, B_FOLLOW_ALL, B_WILL_DRAW | B_FRAME_EVENTS)
{
	BRect bound = Bounds();

	map = new BBitmap(bound, B_RGB32, true);
	aview = new BView(bound, "ColorsWorld_view", B_FOLLOW_ALL, 0);
	map->AddChild(aview);
	map->Lock();
	aview->SetHighColor(0,0,0,255);
	map->Unlock();
	DrawFirst();
}

ColView::ColView(BMessage *msg) : BView(msg)
{
	BRect bound = Bounds();
	map = new BBitmap(bound, B_RGB32, true);
	aview = new BView(bound, "ColorsWorld_view", B_FOLLOW_ALL, 0);
	map->AddChild(aview);
	map->Lock();
	aview->SetHighColor(0,0,0,255); 
	map->Unlock();
	DrawFirst();
}


status_t ColView::Archive(BMessage *data, bool deep) const
{
	return B_ERROR;
	
	WriteHighColor();
	BView::Archive(data);
    data->AddString("class", "ColView");
 	data->AddString("add_on", "application/x-vnd.abisoft-ColorWorld");
	return B_OK;
}

BArchivable *ColView::Instantiate(BMessage *data)
{
	if (!validate_instantiation(data, "ColView"))	
		return NULL;
	return new ColView(data);
}


void ColView::Draw(BRect updateRect)
{
	DrawBitmap(map, B_ORIGIN);	
} 
void ColView::FrameResized(float new_width, float new_height)
{
	if (NULL == Window()->MessageQueue()->FindMessage(B_VIEW_RESIZED, 1))	{
		map->Lock();
		fHighColor = aview->HighColor();
		map->Unlock();
	
		WriteHighColor();
		BRect bound = Bounds();
		map->Lock();
		map->RemoveChild(aview);
		map->Unlock();
		delete map;
		map = new BBitmap(bound, B_RGB32, true);
		aview->ResizeTo(bound.Width(), bound.Height());
		map->AddChild(aview);
		DrawFirst();
		DrawBitmap(map, B_ORIGIN);	
	}
}
void ColView::DrawFirst()
{
	ReadHighColor();
	//draw all of our colors
	map->Lock();
	rgb_color color;
	//lets go for 32x8
	BRect bound = Bounds();
	fillWidth = bound.right / 32;
	fillHeight = bound.bottom / 9; //1/9 for selected color
	BRect fillRect = BRect(0, 0, fillWidth, fillHeight); 

	colorRect.Set(0, bound.Height() - fillHeight, bound.Width(), bound.Height()); 
	
	int line = 0;
	for(int i = 0; i < 256; i++) {
		line++;
		color = BScreen().ColorForIndex(i);
		color.alpha = 255;
		aview->SetHighColor(color);
		aview->FillRect(fillRect);
		if(line%32 != 0)
			fillRect.OffsetBy(fillWidth,0); 
		else
			fillRect.OffsetTo(0,line/32*fillHeight);
	}
	
	
	aview->SetHighColor(fHighColor);
	aview->FillRect(colorRect);
	
	aview->SetHighColor(0,0,0,255);  
	aview->SetHighColor(fHighColor);

	aview->Sync();
	map->Unlock();
	DrawBitmap(map, B_ORIGIN);	
}
void ColView::ChangeColor(rgb_color rgb)
{
	map->Lock();
	rgb_color before = aview->HighColor();
	int32 old = BScreen().IndexForColor(before);
	int32 newer = BScreen().IndexForColor(rgb);
	if(old != newer) { //if we have a new color
		aview->SetHighColor(rgb);
		aview->FillRect(colorRect);
		aview->SetHighColor(0,0,0,255);  
		aview->SetHighColor(rgb);

		aview->Sync();
	}
	fHighColor = aview->HighColor();
	map->Unlock();
	if(old != newer) //if we have a new color
		DrawBitmap(map, B_ORIGIN);
}	


void ColView::MouseDown(BPoint point)
{
	
	ulong buttons;
	GetMouse(&point, &buttons);
	
	rgb_color rgb = word_to_color(GetPixel(map, point));
	
	map->Lock();
	aview->SetHighColor(rgb);
	aview->FillRect(colorRect);
	aview->SetHighColor(0,0,0,255);  
	aview->SetHighColor(rgb);

	aview->Sync();
	map->Unlock();
	
	DrawBitmap(map, B_ORIGIN);
	
	//COLOR_BROADCAST is defined at the beginning of the program.
	//if true it broadcasts the message !CLR to all programs
	//you can support it by copying the code for roColour drops from your
	//BWindow to your BApplication
	
	if(COLOR_BROADCAST) {
  	 	BMessage *colorBroadcast = new BMessage('!CLR');
    	colorBroadcast->AddData( "RGBColor", 
        	   (type_code) B_RGB_COLOR_TYPE, &rgb,  sizeof(rgb));
		be_roster->Broadcast(colorBroadcast);
	}
	
	BBitmap *dragMap = new BBitmap(BRect(0,0,fillWidth,fillHeight), B_RGB32, true);
	BView *dragView = new BView(BRect(0,0,fillWidth,fillHeight), "dragView", B_FOLLOW_NONE, B_WILL_DRAW);
	
	
	dragMap->Lock();
	dragMap->AddChild(dragView);
	dragView->SetHighColor(rgb);
	dragView->FillRect(dragView->Bounds());
	dragView->Sync();
	dragMap->RemoveChild(dragView);
	delete dragView;
	dragMap->Unlock();
	
	BMessage *colorMessage = new BMessage(B_PASTE);
	colorMessage->AddData( "RGBColor", 
        	   (type_code) B_RGB_COLOR_TYPE, &rgb,  sizeof(rgb));
	
	char *text = RGB2Text(&rgb);
	
	colorMessage->AddData("text/plain",B_MIME_TYPE,text,strlen(text)+1);
	
	DragMessage(colorMessage, dragMap, B_OP_BLEND, B_ORIGIN);
}

void ColView::ReadHighColor()
{
	BPath path;
	
	find_directory(B_USER_SETTINGS_DIRECTORY, &path, true);
	
	BDirectory dir(path.Path());
	BFile file;
	
	path.Append("ColorWorldSettings");
	dir.CreateFile(path.Path(), &file);
	
	if(file.ReadAttr("Color", B_RGB_COLOR_TYPE, 0, 
		&fHighColor, sizeof(rgb_color)) != sizeof(rgb_color)) {
		fHighColor.red = fHighColor.green = fHighColor.blue = 0;
		fHighColor.alpha = 255;
	}
}

void ColView::WriteHighColor() const
{
	BPath path;
	
	find_directory(B_USER_SETTINGS_DIRECTORY, &path, true);
	
	BDirectory dir(path.Path());
	BFile file;
	
	path.Append("ColorWorldSettings");
	dir.CreateFile(path.Path(), &file);
	
	file.WriteAttr("Color", B_RGB_COLOR_TYPE, 0, 
		&fHighColor, sizeof(rgb_color));
}
void ColView::QuitRequested()
{
	map->Lock();
	fHighColor = aview->HighColor();
	map->Unlock();
	
	WriteHighColor();
}	

// This code is *cough* borrowed from DropColour. Just a little function
// to convert the rgb_color to a string like #FF0F9A
// I made a few modifications to suite my own purposes.

char *ColView::RGB2Text(rgb_color* color) {
	static char* p="0123456789ABCDEF";

	char *rgbtxt = new char[8];

	rgbtxt[0]='#';
	rgbtxt[1]=p[(color->red & 0xF0)>>4];
	rgbtxt[2]=p[(color->red & 0x0F)];
	rgbtxt[3]=p[(color->green & 0xF0)>>4];
	rgbtxt[4]=p[(color->green & 0x0F)];
	rgbtxt[5]=p[(color->blue & 0xF0)>>4];
	rgbtxt[6]=p[(color->blue & 0x0F)];
	rgbtxt[7]='\0';

	return rgbtxt;
}

void ColView::MessageReceived(BMessage *message)
{
	
	rgb_color *rgbColor;
	ssize_t numBytes = sizeof(struct rgb_color);
	
	//easy code to use for receiving ColorWorld and
	//roColour drops
	if ( message->FindData( "RGBColor", 
        	   (type_code) B_RGB_COLOR_TYPE, (const void **)  &rgbColor,  &numBytes ) == B_OK)	{
     			if ( message->WasDropped() )  { 
					ChangeColor(*rgbColor);
				} 
	}
	switch(message->what)	{
	
		default:
			BView::MessageReceived(message);
			break;
	}
}

