/* ColView.h
	Copyright (C) 1996-99, Seth Flaxman (seth@abisoft.com)
	Permission to use, copy, modify, and distribute this software for any
	purpose without fee is hereby granted, provided that this entire notice
	is included in all copies of any software which is or includes a copy
	or modification of this software and in all copies of the supporting
	documentation for such software.
	THIS SOFTWARE IS BEING PROVIDED "AS IS", WITHOUT ANY EXPRESS OR IMPLIED
	WARRANTY.  IN PARTICULAR, THE AUTHOR DOES NOT MAKE ANY
	REPRESENTATION OR WARRANTY OF ANY KIND CONCERNING THE MERCHANTABILITY
	OF THIS SOFTWARE OR ITS FITNESS FOR ANY PARTICULAR PURPOSE.
*/

#ifndef _COLVIEW_H_
#define _COLVIEW_H_

#include <View.h>
#include <Dragger.h>
#include <Bitmap.h>

class _EXPORT ColView : public BView {

public:
	ColView(BRect frame, char *name);
	
	// replicant stuff
	ColView(BMessage *msg);
	virtual status_t Archive(BMessage *message, bool deep = true) const;
	static	BArchivable *Instantiate(BMessage *data);
	
	virtual void MouseDown(BPoint point);
	virtual void Draw(BRect updateRect);
	void ChangeColor(rgb_color rgb);
	virtual void FrameResized(float new_width, float new_height);
	virtual void QuitRequested();
protected:
	virtual void ReadHighColor();
	virtual void WriteHighColor() const;
	virtual void DrawFirst();
	virtual char *RGB2Text(rgb_color *value);
	virtual void MessageReceived(BMessage *message);
private:
	BBitmap *map, *dragBitmap;
	BView *aview;
	BRect colorRect;
	rgb_color fHighColor;
	float fillWidth, fillHeight;
};

// This code is copied from some examples by Jonathan Perret,
// and it may bear his copyright (?) 
// I think you can use it though.

uint32 GetPixel(BBitmap *bitmap, BPoint point);
inline rgb_color word_to_color(uint32 word);

inline uint32 GetPixel(BBitmap *bitmap, BPoint point)
{
	return ((uint32 *)(bitmap->Bits()))[int32(point.y)*bitmap->BytesPerRow()/4+int32(point.x)];
}

#if B_HOST_IS_BENDIAN
inline uint32 color_to_word(rgb_color col)
	{ return ((uint32)col.blue<<24)
		|((uint32)col.green<<16)
		|((uint32)col.red<<8)
		|(uint32)col.alpha; };
	
inline rgb_color word_to_color(uint32 word)
	{ rgb_color col={ word>>8, word>>16, word>>24, word }; 
		return col; }
#else /* B_HOST_IS_BENDIAN */
/* note : words are RGBA in registers ! */
inline uint32 color_to_word(rgb_color col)
	{ return ((uint32)col.blue)
		|((uint32)col.green<<8)
		|((uint32)col.red<<16)
		|(uint32)col.alpha<<24; };
	
inline rgb_color word_to_color(uint32 word)
	{ rgb_color col={ word>>16, word>>8, word, word>>24 };
		return col; }
#endif /* B_HOST_IS_BENDIAN */


const int32 kDataWidth = 18;
const int32 kDataHeight = 18;
const color_space kDataColorSpace = B_CMAP8;

const uint8 kDataBits [] = {
	0x98,0x98,0x98,0xff,0x98,0x98,0x98,0xff,0x98,0x98,0x98,0xff,0x98,0x98,0x98,0xff,0x98,0x98,0x98,0xff,0x98,0x98,0x98,0xff,0x98,0x98,0x98,0xff,0x98,0x98,0x98,0xff,0x98,0x98,0x98,0xff,0x98,0x98,0x98,0xff,0x98,0x98,0x98,0xff,0x98,0x98,0x98,0xff,
	0x98,0x98,0x98,0xff,0x98,0x98,0x98,0xff,0x98,0x98,0x98,0xff,0xff,0xff,0xff,0xff,0xd8,0xd8,0xd8,0xff,0xb8,0xb8,0xb8,0xff,0x60,0x60,0x60,0xff,0x98,0x98,0x98,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,
	0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xd8,0xd8,0xd8,0xff,
	0xb8,0xb8,0xb8,0xff,0x60,0x60,0x60,0xff,0x98,0x98,0x98,0xff,0xff,0xff,0xff,0xff,0xd8,0xd8,0xd8,0xff,0xd8,0xd8,0xd8,0xff,0xd8,0xd8,0xd8,0xff,0xd8,0xd8,0xd8,0xff,0xd8,0xd8,0xd8,0xff,0xd8,0xd8,0xd8,0xff,0xd8,0xd8,0xd8,0xff,0xd8,0xd8,0xd8,0xff,
	0xd8,0xd8,0xd8,0xff,0xd8,0xd8,0xd8,0xff,0xd8,0xd8,0xd8,0xff,0xd8,0xd8,0xd8,0xff,0xd8,0xd8,0xd8,0xff,0xd8,0xd8,0xd8,0xff,0xd8,0xd8,0xd8,0xff,0xb8,0xb8,0xb8,0xff,0x60,0x60,0x60,0xff,0x98,0x98,0x98,0xff,0xff,0xff,0xff,0xff,0xd8,0xd8,0xd8,0xff,
	0xd8,0xd8,0xd8,0xff,0xd8,0xd8,0xd8,0xff,0xd8,0xd8,0xd8,0xff,0xd8,0xd8,0xd8,0xff,0xd8,0xd8,0xd8,0xff,0xd8,0xd8,0xd8,0xff,0xd8,0xd8,0xd8,0xff,0xd8,0xd8,0xd8,0xff,0xd8,0xd8,0xd8,0xff,0xd8,0xd8,0xd8,0xff,0xd8,0xd8,0xd8,0xff,0xd8,0xd8,0xd8,0xff,
	0xd8,0xd8,0xd8,0xff,0xd8,0xd8,0xd8,0xff,0xb8,0xb8,0xb8,0xff,0x60,0x60,0x60,0xff,0x98,0x98,0x98,0xff,0xff,0xff,0xff,0xff,0xd8,0xd8,0xd8,0xff,0xd8,0xd8,0xd8,0xff,0xd8,0xd8,0xd8,0xff,0xd8,0xd8,0xd8,0xff,0xd8,0xd8,0xd8,0xff,0xd8,0xd8,0xd8,0xff,
	0xd8,0xd8,0xd8,0xff,0xd8,0xd8,0xd8,0xff,0xd8,0xd8,0xd8,0xff,0xd8,0xd8,0xd8,0xff,0xd8,0xd8,0xd8,0xff,0x60,0x60,0x60,0xff,0xd8,0xd8,0xd8,0xff,0xd8,0xd8,0xd8,0xff,0xd8,0xd8,0xd8,0xff,0xb8,0xb8,0xb8,0xff,0x60,0x60,0x60,0xff,0x98,0x98,0x98,0xff,
	0xff,0xff,0xff,0xff,0xd8,0xd8,0xd8,0xff,0xd8,0xd8,0xd8,0xff,0xd8,0xd8,0xd8,0xff,0xd8,0xd8,0xd8,0xff,0xd8,0xd8,0xd8,0xff,0xd8,0xd8,0xd8,0xff,0xd8,0xd8,0xd8,0xff,0xd8,0xd8,0xd8,0xff,0xd8,0xd8,0xd8,0xff,0xd8,0xd8,0xd8,0xff,0xd8,0xd8,0xd8,0xff,
	0xd8,0xd8,0xd8,0xff,0xff,0xff,0xff,0xff,0xd8,0xd8,0xd8,0xff,0xd8,0xd8,0xd8,0xff,0xb8,0xb8,0xb8,0xff,0x60,0x60,0x60,0xff,0x98,0x98,0x98,0xff,0xff,0xff,0xff,0xff,0xd8,0xd8,0xd8,0xff,0xd8,0xd8,0xd8,0xff,0xd8,0xd8,0xd8,0xff,0xd8,0xd8,0xd8,0xff,
	0xd8,0xd8,0xd8,0xff,0xd8,0xd8,0xd8,0xff,0xd8,0xd8,0xd8,0xff,0xd8,0xd8,0xd8,0xff,0xd8,0xd8,0xd8,0xff,0xd8,0xd8,0xd8,0xff,0xd8,0xd8,0xd8,0xff,0xd8,0xd8,0xd8,0xff,0xd8,0xd8,0xd8,0xff,0xd8,0xd8,0xd8,0xff,0xd8,0xd8,0xd8,0xff,0xb8,0xb8,0xb8,0xff,
	0x60,0x60,0x60,0xff,0x98,0x98,0x98,0xff,0xff,0xff,0xff,0xff,0xd8,0xd8,0xd8,0xff,0xd8,0xd8,0xd8,0xff,0xd8,0xd8,0xd8,0xff,0xd8,0xd8,0xd8,0xff,0xd8,0xd8,0xd8,0xff,0xd8,0xd8,0xd8,0xff,0xd8,0xd8,0xd8,0xff,0xd8,0xd8,0xd8,0xff,0x60,0x60,0x60,0xff,
	0xd8,0xd8,0xd8,0xff,0xd8,0xd8,0xd8,0xff,0x60,0x60,0x60,0xff,0xd8,0xd8,0xd8,0xff,0xd8,0xd8,0xd8,0xff,0xd8,0xd8,0xd8,0xff,0xb8,0xb8,0xb8,0xff,0x60,0x60,0x60,0xff,0x98,0x98,0x98,0xff,0xff,0xff,0xff,0xff,0xd8,0xd8,0xd8,0xff,0xd8,0xd8,0xd8,0xff,
	0xd8,0xd8,0xd8,0xff,0xd8,0xd8,0xd8,0xff,0xd8,0xd8,0xd8,0xff,0xd8,0xd8,0xd8,0xff,0xd8,0xd8,0xd8,0xff,0xd8,0xd8,0xd8,0xff,0xd8,0xd8,0xd8,0xff,0xff,0xff,0xff,0xff,0xd8,0xd8,0xd8,0xff,0xd8,0xd8,0xd8,0xff,0xff,0xff,0xff,0xff,0xd8,0xd8,0xd8,0xff,
	0xd8,0xd8,0xd8,0xff,0xb8,0xb8,0xb8,0xff,0x60,0x60,0x60,0xff,0x98,0x98,0x98,0xff,0xff,0xff,0xff,0xff,0xd8,0xd8,0xd8,0xff,0xd8,0xd8,0xd8,0xff,0xd8,0xd8,0xd8,0xff,0xd8,0xd8,0xd8,0xff,0xd8,0xd8,0xd8,0xff,0xd8,0xd8,0xd8,0xff,0xd8,0xd8,0xd8,0xff,
	0xd8,0xd8,0xd8,0xff,0xd8,0xd8,0xd8,0xff,0xd8,0xd8,0xd8,0xff,0xd8,0xd8,0xd8,0xff,0xd8,0xd8,0xd8,0xff,0xd8,0xd8,0xd8,0xff,0xd8,0xd8,0xd8,0xff,0xd8,0xd8,0xd8,0xff,0xb8,0xb8,0xb8,0xff,0x60,0x60,0x60,0xff,0x98,0x98,0x98,0xff,0xff,0xff,0xff,0xff,
	0xd8,0xd8,0xd8,0xff,0xd8,0xd8,0xd8,0xff,0xd8,0xd8,0xd8,0xff,0xd8,0xd8,0xd8,0xff,0xd8,0xd8,0xd8,0xff,0x60,0x60,0x60,0xff,0xd8,0xd8,0xd8,0xff,0xd8,0xd8,0xd8,0xff,0x60,0x60,0x60,0xff,0xd8,0xd8,0xd8,0xff,0xd8,0xd8,0xd8,0xff,0x60,0x60,0x60,0xff,
	0xd8,0xd8,0xd8,0xff,0xd8,0xd8,0xd8,0xff,0xd8,0xd8,0xd8,0xff,0xb8,0xb8,0xb8,0xff,0x60,0x60,0x60,0xff,0x98,0x98,0x98,0xff,0xff,0xff,0xff,0xff,0xd8,0xd8,0xd8,0xff,0xd8,0xd8,0xd8,0xff,0xd8,0xd8,0xd8,0xff,0xd8,0xd8,0xd8,0xff,0xd8,0xd8,0xd8,0xff,
	0xd8,0xd8,0xd8,0xff,0xff,0xff,0xff,0xff,0xd8,0xd8,0xd8,0xff,0xd8,0xd8,0xd8,0xff,0xff,0xff,0xff,0xff,0xd8,0xd8,0xd8,0xff,0xd8,0xd8,0xd8,0xff,0xff,0xff,0xff,0xff,0xd8,0xd8,0xd8,0xff,0xd8,0xd8,0xd8,0xff,0xb8,0xb8,0xb8,0xff,0x60,0x60,0x60,0xff,
	0x98,0x98,0x98,0xff,0xff,0xff,0xff,0xff,0xd8,0xd8,0xd8,0xff,0xd8,0xd8,0xd8,0xff,0xd8,0xd8,0xd8,0xff,0xd8,0xd8,0xd8,0xff,0xd8,0xd8,0xd8,0xff,0xd8,0xd8,0xd8,0xff,0xd8,0xd8,0xd8,0xff,0xd8,0xd8,0xd8,0xff,0xd8,0xd8,0xd8,0xff,0xd8,0xd8,0xd8,0xff,
	0xd8,0xd8,0xd8,0xff,0xd8,0xd8,0xd8,0xff,0xd8,0xd8,0xd8,0xff,0xd8,0xd8,0xd8,0xff,0xd8,0xd8,0xd8,0xff,0xb8,0xb8,0xb8,0xff,0x60,0x60,0x60,0xff,0x98,0x98,0x98,0xff,0xff,0xff,0xff,0xff,0xd8,0xd8,0xd8,0xff,0xd8,0xd8,0xd8,0xff,0x60,0x60,0x60,0xff,
	0xd8,0xd8,0xd8,0xff,0xd8,0xd8,0xd8,0xff,0x60,0x60,0x60,0xff,0xd8,0xd8,0xd8,0xff,0xd8,0xd8,0xd8,0xff,0x60,0x60,0x60,0xff,0xd8,0xd8,0xd8,0xff,0xd8,0xd8,0xd8,0xff,0x60,0x60,0x60,0xff,0xd8,0xd8,0xd8,0xff,0xd8,0xd8,0xd8,0xff,0xd8,0xd8,0xd8,0xff,
	0xb8,0xb8,0xb8,0xff,0x60,0x60,0x60,0xff,0x98,0x98,0x98,0xff,0xff,0xff,0xff,0xff,0xd8,0xd8,0xd8,0xff,0xd8,0xd8,0xd8,0xff,0xd8,0xd8,0xd8,0xff,0xff,0xff,0xff,0xff,0xd8,0xd8,0xd8,0xff,0xd8,0xd8,0xd8,0xff,0xff,0xff,0xff,0xff,0xd8,0xd8,0xd8,0xff,
	0xd8,0xd8,0xd8,0xff,0xff,0xff,0xff,0xff,0xd8,0xd8,0xd8,0xff,0xd8,0xd8,0xd8,0xff,0xff,0xff,0xff,0xff,0xd8,0xd8,0xd8,0xff,0xd8,0xd8,0xd8,0xff,0xb8,0xb8,0xb8,0xff,0x60,0x60,0x60,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xd8,0xd8,0xd8,0xff,
	0xd8,0xd8,0xd8,0xff,0xd8,0xd8,0xd8,0xff,0xd8,0xd8,0xd8,0xff,0xd8,0xd8,0xd8,0xff,0xd8,0xd8,0xd8,0xff,0xd8,0xd8,0xd8,0xff,0xd8,0xd8,0xd8,0xff,0xd8,0xd8,0xd8,0xff,0xd8,0xd8,0xd8,0xff,0xd8,0xd8,0xd8,0xff,0xd8,0xd8,0xd8,0xff,0xd8,0xd8,0xd8,0xff,
	0xd8,0xd8,0xd8,0xff,0xd8,0xd8,0xd8,0xff,0xb8,0xb8,0xb8,0xff,0x60,0x60,0x60,0xff,0xd8,0xd8,0xd8,0xff,0xd8,0xd8,0xd8,0xff,0xd8,0xd8,0xd8,0xff,0xd8,0xd8,0xd8,0xff,0xd8,0xd8,0xd8,0xff,0xd8,0xd8,0xd8,0xff,0xd8,0xd8,0xd8,0xff,0xd8,0xd8,0xd8,0xff,
	0xd8,0xd8,0xd8,0xff,0xd8,0xd8,0xd8,0xff,0xd8,0xd8,0xd8,0xff,0xd8,0xd8,0xd8,0xff,0xd8,0xd8,0xd8,0xff,0xd8,0xd8,0xd8,0xff,0xd8,0xd8,0xd8,0xff,0xd8,0xd8,0xd8,0xff,0xd8,0xd8,0xd8,0xff,0xb8,0xb8,0xb8,0xff,0x60,0x60,0x60,0xff,0xb8,0xb8,0xb8,0xff,
	0xb8,0xb8,0xb8,0xff,0xb8,0xb8,0xb8,0xff,0xb8,0xb8,0xb8,0xff,0xb8,0xb8,0xb8,0xff,0xb8,0xb8,0xb8,0xff,0xb8,0xb8,0xb8,0xff,0xb8,0xb8,0xb8,0xff,0xb8,0xb8,0xb8,0xff,0xb8,0xb8,0xb8,0xff,0xb8,0xb8,0xb8,0xff,0xb8,0xb8,0xb8,0xff,0xb8,0xb8,0xb8,0xff,
	0xb8,0xb8,0xb8,0xff,0xb8,0xb8,0xb8,0xff,0xb8,0xb8,0xb8,0xff,0xb8,0xb8,0xb8,0xff,0xb8,0xb8,0xb8,0xff,0x60,0x60,0x60,0xff,0x60,0x60,0x60,0xff,0x60,0x60,0x60,0xff,0x60,0x60,0x60,0xff,0x60,0x60,0x60,0xff,0x60,0x60,0x60,0xff,0x60,0x60,0x60,0xff,
	0x60,0x60,0x60,0xff,0x60,0x60,0x60,0xff,0x60,0x60,0x60,0xff,0x60,0x60,0x60,0xff,0x60,0x60,0x60,0xff,0x60,0x60,0x60,0xff,0x60,0x60,0x60,0xff,0x60,0x60,0x60,0xff,0x60,0x60,0x60,0xff,0x60,0x60,0x60,0xff,0x60,0x60,0x60,0xff,0x60,0x60,0x60,0xff,
	0x60,0x60,0x60,0xff
};



#endif //_COLVIEW_H_